#!/usr/local/bin/perl -w

# filter.pl - version 0.2


# This program takes as input the file (output of type.pl). This file
# has type based gold data. This program filters out the phrasal
# translations for the type based gold data.

# Copyright (C) 2001-2002.
# Ted Perdersen, University Of Minnesota, Duluth.
# tpederse@d.umn.edu
# Nitin Varma, University Of Minnesota, Duluth.
# varm0003@d.umn.edu


# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# package required for command line options
use Getopt::Long;

GetOptions("version", "help");

# Now strip the command line options

if ( defined $opt_help ) { $help = $opt_help; }
else                     { $help = 0; }


if ( defined $opt_version ) { $version = $opt_version; }
else                        { $version = 0; } 


# if asked for help, print the help and exit.  
if ( $help )
{
    &printHelp();
    exit;
}

# if asked for version, print the version number and exit
if ( $version )
{
    &printVersion();
    exit;
}

if($#ARGV < 2)
{
  
    &printHelp();
    exit;
}

$input = shift;
$output1 = shift;
$output2 = shift;


open(FILE, "$input") || die "error opening file $input\n";
open(FILE1, ">$output1") || die "error creating $output1\n";
open(FILE2, ">$output2") || die "error creating file $output2\n";

if( (-e $output1 ) )
  {
       print "\n$output1 exists !!!  ";
       print "\nDo you want to overwrite it (Y/N)?";
       $ans = <STDIN>;
       chomp $ans;
       $ans = uc $ans;
       if($ans ne "Y")
       {
	   exit;
       }
     
  }

if( (-e $output2 ) )
  {
       print "\n$output2 exists !!!  ";
       print "\nDo you want to overwrite it (Y/N)?";
       $ans = <STDIN>;
       chomp $ans;
       $ans = uc $ans;
       if($ans ne "Y")
       {
	   exit;
       }
     
  }

while($line = <FILE>)
{

    ($leftside, $rightside) = split(/ -> /,$line);
    $match = $leftside.$rightside;
   


    if($match =~m/ /)
    {
	print FILE2 "$line";
    }
    else
    {
      print FILE1 "$line";    
  }
}
close(FILE1);
close(FILE2);
close(FILE); 

# ---------------------------------------------------------------------------
#                                 USER DEFINED FUNCTIONS
# ---------------------------------------------------------------------------

# prints what are the command line options for the program and explain what
# input is required by different arguments
sub printHelp{

    &printUsage();
    print "\n\nOPTIONS:";
    print "\n\n  --version     Prints the version number";
    print "\n\n  --help        Prints the help message\n";
}



# prints the usage of the program.
sub printUsage{

 print "\nUsage:filter.pl [ { --help | --version } FILE1 FILE2 FILE3\n";
 print "\nFILE1 - The file created by tpye.pl and input to filter.pl\n";
 print "FILE2 - Output file of filter.pl containing only word to word translations \n";
 print "i.e no phrasal translations\n";
 print "FILe3 - Output file of filter.pl containing only the phrasal translations\n";
}




# prints the version number and copyright information.
sub printVersion{

    print "filter.pl -  Version 0.2\n";
    print "Copyright (C) 2001-2002, Ted Pedersen & Nitin Varma\n";

}

