#!/usr/local/bin/perl
# eval.pl - version 0.2

# Program for comparing the compiled bilingual lexicon with the
# the gold standard data.

# Copyright (C) 2001-2002.
# Ted Perdersen, University Of Minnesota, Duluth.
# tpederse@d.umn.edu
# Nitin Varma, University Of Minnesota, Duluth.
# varm0003@d.umn.edu


# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# ----------------------------------------------------------------------------
#                             Main Program
# ----------------------------------------------------------------------------

# package required for command line options
use Getopt::Long;

GetOptions("version", "help", "list=s");

# Now strip the command line options

if ( defined $opt_help ) { $help = $opt_help; }
else                     { $help = 0; }


if ( defined $opt_version ) { $version = $opt_version; }
else                        { $version = 0; } 

if ( defined $opt_list ) { open(LIST, ">$opt_list"); }

# if asked for help, print the help and exit.  
if ( $help )
{
    &printHelp();
    exit;
}

# if asked for version, print the version number and exit
if ( $version )
{
    &printVersion();
    exit;
}




# check if all 2 non-optional arguments are provided or not.
if($#ARGV < 1 )
{
    
    &printUsage();
    exit;
}

# Done with the command line options !!!
# Now find the source, target and output files
$GOLDFILE =  shift;
$OURFILE = shift;

  if( ! (-e $GOLDFILE ) )
  {
      print "\nERROR!!! $GOLDFILE not found\n ";
      exit;
     
  }
   
    if( ! (-e $OURFILE ) )
  {
      print "\nERROR!!! $OURFILE not found\n ";
      exit;
     
  }

$goldCount =0;
$ourCount = 0;
$matchFound = 0;

open(FILE, $GOLDFILE);
while($line = <FILE>)
{
    $line =~ s/^\s+//;
    $line =~ s/\s+$//;
    $line =~ s/\s+/ /;
    $line =~ s/\n+//;
    if(!($line =~ m/null/))
    {
	$goldHash{$line}++;
        $goldCount++;
    }
    
}
close(FILE);
open(FILE, $OURFILE);
$line  = <FILE>;

$ourCount = 0;
while($line = <FILE>)
{ 
    $line =~ s/^\s+//;
    $line =~ s/\s+$//;
    $line =~ s/\s+/ /;
    $line =~ s/\n+//;   
    ($words, $rest) = split(/ /, $line);
    $words =~ s/<>/ /g;
    ($word1, $word2) = split(/ /,$words); 
    $pair = $word1." -> ".$word2;
    $ourHash{$pair}++;
    $ourCount++;
}

$matchFound = 0;
foreach $pair(keys %ourHash)
  {
      if(exists $goldHash{$pair})
      {

	  $matchFound++;
          $check = "CORRECT";
            
      }
      else
      {
	  $check = "NOT CORRECT";
      }
    if(defined opt_list)
    {
	print LIST "$pair $check\n";
    }
  }
    
close(LIST);
$F_measure=0;
$precision=0;
$recall = 0;
print "\nGold data g=$goldCount";
print "\nTotal found o=$ourCount";
print "\nTotal Correct m=$matchFound";

if($matchFound !=0)
{
    $precision = $matchFound / $ourCount;
    $recall = $matchFound / $goldCount;
    $F_measure = (2*$precision*$recall) / ($recall + $precision);
}


printf "\nPrecision p=%4.2f",$precision;
printf "\nRecall r=%4.2f",$recall;
printf "\nF-measure f=%4.2f\n",$F_measure;



# ---------------------------------------------------------------------------
#                                 USER DEFINED FUNCTIONS
# ---------------------------------------------------------------------------

# prints what are the command line options for the program and explain what
# input is required by different arguments
sub printHelp{

    &printUsage();
    print "\n\nOPTIONS:";
    print "\n\n  --version     Prints the version number";
    print "\n\n  --help        Prints the help message";
    print "\n\n  --LIST FILE   prints into a FILE whether an entry in the lexicon\n";
    print "                being evaluated is right or wrong\n";
                                
}



# prints the usage of the program.
sub printUsage{

 print "Usage:eval.pl [ { --help | --version } DATAFILE1 DATAFILE2 ]\n";
 print "\nDATAFILE1 is the GOLD_DATA";
 print "\nDATAFILE2 is the OUTPUT of statistic.pl or ensemble.pl program\n";
}




# prints the version number and copyright information.
sub printVersion{

    print "eval.pl -  Version 0.2\n";
    print "Copyright (C) 2001-2002, Ted Pedersen & Nitin Varma\n";


}
 
