#!/usr/local/bin/perl -w

# kvec.pl - version 0.2

# This program is re-implementation of part of k-vec algorithm to align the 
# text of two languages. It finds out the word correspondences in two texts.

# Copyright (C) 2001-2002.
# Ted Perdersen, University Of Minnesota, Duluth.
# tpederse@d.umn.edu
# Nitin Varma, University Of Minnesota, Duluth.
# varm0003@d.umn.edu


# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


# ----------------------------------------------------------------------------
#                             Main Program
# ----------------------------------------------------------------------------

# package required for command line options
use Getopt::Long;

$options = 0;
GetOptions("version", "help", "pieces=i", "lcutOff=i", "ucutOff=i");

# Now strip the command line options
if ( defined $opt_pieces ) { $pieces = $opt_pieces; 
                             $options++; 
                           } 

if ( defined $opt_lcutOff ) { $lcutOff = $opt_lcutOff;
                             $options++;
                           }
else                       { $lcutOff = 4; }

if ( defined $opt_ucutOff ) { 

                              $ucutOff = $opt_ucutOff;

                           }
else                       {  $ucutOff = 10000; }

if ( defined $opt_help ) { $help = $opt_help; }
else                     { $help = 0; }


if ( defined $opt_version ) { $version = $opt_version; }
else                        { $version = 0; } 

# if asked for help, print the help and exit.  
if ( $help )
{
    &printHelp();
    exit;
}

# if asked for version, print the version number and exit
if ( $version )
{
    &printVersion();
    exit;
}


# check if all 3 non-optional arguments are provided or not.
if($#ARGV < 1 )
{
    
    &printUsage();
    exit;
}


# Done with the command line options !!!
# Now find the source, target and result files
$targetFile = shift;
$sourceFile = shift;


# Finishes stripping all the command line arguments.
# Now starts the actual work....

# calculate the total words in both source and target files
$sourceWords =  &calculateWords($sourceFile,\%sourceHash);
$targetWords =  &calculateWords($targetFile,\%targetHash); 


# delete the words from the hash below the cutoff frequency(default = 4)
#%sourceHash = &deleteHash(%sourceHash);
#%targetHash = &deleteHash(%targetHash);


# Depending on whether no of pieces was given as an command line option or not
# calculate number of pieces, number of words in each piece for both the source
# and target file.

if ( defined $pieces )
{
    if($pieces == 0)
    {

    print "Number of pieces should be greater than zero\n";
    exit;
    }
    $sourcePieces = $pieces;
    $targetPieces = $pieces;
    $Source_words_per_piece = $sourceWords / $sourcePieces;
    $Target_words_per_piece = $targetWords / $targetPieces;
  
}

else 
{
   $sourcePieces = sqrt( $sourceWords );
   $sourcePieces = &floorCeil($sourcePieces);
   $Source_words_per_piece = $sourceWords / $sourcePieces;
   $targetPieces = $sourcePieces;
   $Target_words_per_piece = $targetWords/$targetPieces;
} 

$pieces = $sourcePieces;
$Source_words_per_piece = &floorCeil($Source_words_per_piece);
$Target_words_per_piece = &floorCeil($Target_words_per_piece);


&checkPiecevalue();

# call the function createPieces which takes the source file and divides it 
# pieces Do  this for both the source and target file.
&createPieces(\@sourceArray,$sourceFile,$Source_words_per_piece,$sourcePieces);
&createPieces(\@targetArray,$targetFile,$Target_words_per_piece,$targetPieces);

# Pass the reference of the hash of the piece from the source text and the
# reference of the corresponding piece of the target text.
for($i=0;$i<$pieces;$i++) 
{
    $hrefSource = $sourceArray[$i];
    $hrefTarget = $targetArray[$i];
&makePairs($hrefSource, $hrefTarget, \%pairsHash, \%sourceWords, \%targetWords);
}

# Call a function to print all possible pairs and the required values.
&printTofile(\%pairsHash, \%sourceWords, \%targetWords); 


# ---------------------------------------------------------------------------
#                                 USER DEFINED FUNCTIONS
# ---------------------------------------------------------------------------

# prints what are the command line options for the program and explain what
# input is required by different arguments
sub printHelp{

    &printUsage();
    print "OPTIONS:";
    print "\n\n  --lcutoff N   Ignores all the word pairs with frequency < N.";
    print "\n                By default N = 4";
    print "\n\n  --ucutoff M   Ignores all the word pairs with frequency > M.";
    print "\n                By default it considers all the words above lcutoff";
    print "\n\n  --version     Prints the version number";
    print "\n\n  --help        Prints the help message";
    print "\n\n  --pieces K    Allows to specify the number of pieces you";
    print "\n                want to divide the text into. By default K is";
    print "\n                set to square root of the total number of words"; 
    print "\n                in the source file.\n"; 

 
}



# prints the usage of the program.
sub printUsage{

 print "Usage:kvec.pl [{[--pieces K --lcutoff N --ucutoff M] TARGET SOURCE |--help|--version}]\n";
}




# prints the version number and copyright information.
sub printVersion{

    print "kvec.pl -  Version 0.2\n";
    print "Copyright (C) 2001-2002, Ted Pedersen & Nitin Varma\n";

}



sub checkPiecevalue{


if($pieces > $sourceWords)
{
    print "Number of pieces greater than number of words in Source file";
    print "\nPlease try again!!!\n";
    exit;
}

if($pieces > $targetWords)
{
    print "Number of pieces greater than number of words in Target file";
    print "\nPlease try again!!!\n";
    exit;
}


}
# this functions takes file as input and finds the total number of words
# in the file.
sub calculateWords{

   
    my $line;
    my $wordCount;
    my $word;
    my $fileName = $_[0];
    my $refHash = $_[1];  
  
# checks if the file exists !! 
     if( ! (-e $fileName ) )
  {
      print "\nERROR!!! $fileName not found\n ";
      exit;
     
  }
   
  
open(FILE, $fileName) || die "Error opening $fileName";
  while( $line = <FILE>)
  {

     $line = &removeSpaces($line);
      $line =~ tr/A-Z/a-z/;
     @wordArray = split(/\s+/,$line);
     $wordCount += @wordArray;
     foreach $word(@wordArray)
     {
	$refHash->{$word}++; 
     }  
     undef(@wordArray);
  }  
    close(FILE);  
    return($wordCount);
    
    
}  

  
# This functions takes a scalar variable as a parameter and replaces the 
# leading and trailing spaces by null and also replaces the multiple spaces by
# single space. Also it removes other punctuations like '.', '?', ';', etc...
sub removeSpaces{
 
  $_[0] =~ tr/A-Z/a-z/;
  #$_[0] =~ s/[\/\|\(\)\[\]\{\}\^\$\*\+\?\.\,\"\-\\\:0-9%]//g;
  $_[0] =~ s/^\s+//g;
  $_[0] =~ s/\s+$//g;
  $_[0] =~ s/\s+/ /g;
  $_[0] =~ s/\n+//g;
  return $_[0];

  
}



# This function takes a hash as input where the key is the word in the file
# and value is the frequency of that word. This function deleted all the words
# from the hash which have frequency less than cutoff
sub deleteHash{

    my $word;
    my %tempHash = @_;
        foreach $word( sort keys %tempHash )
    {
	if( $tempHash{$word} < $lcutOff )
        {
	    delete($tempHash{$word});
        }

     
        elsif( $tempHash{$word} > $ucutOff )
        {
	    delete($tempHash{$word});
        }
    

     } 
       return %tempHash;
 
}




# This function takes as an input an number and rounds it to integer value.
sub floorCeil{

    my $x=0;
    $x = sprintf(" %d ", $_[0]);
    return $x;
}



# This functions creates a an array of hashes such that the first hash contains
# all distinct words in first piece, second hash contains all distinct words
# in second piece and so on ...
sub createPieces{
  
  my $pieceNumber = 0;
  my $totalWords = 0;
  my $refArray = $_[0];
  my $fileName = $_[1];
  my $length = $_[2];
  my $totalPieces = $_[3];
  my @wordArray = ();
  my $line;
  my $p=0;
 
 
  open(FILE, $fileName ) || die "Error opening $fileName";
  while ( $line = <FILE> )
  {

      $line =~ tr/A-Z/a-z/;
     $line = &removeSpaces($line);

# stores all the space seperated words in an array
      @wordArray = (@wordArray, split(/ /,$line ) ); 


      while($#wordArray >= $length)
      {

#if loop added on 12-28-2001
          if($pieceNumber >= $totalPieces)
         {
	     last;
	 }
#if loop added on 12-28-2001
	  for($p=0;$p<$length;$p++)
         {  
             
	    ($out, @wordArray) = @wordArray;
	    $refArray->[$pieceNumber]{$out}++
               
         }
         
         $pieceNumber++;
      
      }
  }


# Once the whole file is read, it just concatenates all the words remaining
# in the array to the last piece and also stores each word in the hash

  if($pieceNumber >= $pieces)
   {
       $pieceNumber--;
   }

   foreach $out(@wordArray)
   {
        
       $refArray->[$pieceNumber]{$out}++;
   }

  close(FILE);
}

# This function takes as input reference of two hashes containing words from 
# the source piece and words from the corresponding target piece. It then 
# takes a word from the source hash and checks if it is above cutoff 
# frequency. If it is then it increements the count for that words in hash. It
# then takes each word from the target hash and for each word above the cutoff
# frequency it increements the count for that word as well as forms a pair of 
# the words and puts it into the pair hash. 
sub makePairs{

    my $hrefSourcehash = $_[0];
    my $hrefTargethash = $_[1];
    my $refPairs = $_[2];
    my $refSourcewords = $_[3];
    my $refTargetwords = $_[4];
    my $word1;
    my $word2;
    my $pair;
    my %temphash;
    foreach $word1(keys %$hrefSourcehash)
    {
        
	if(exists $sourceHash{$word1})
        {
	    $refSourcewords->{$word1}++;
	    foreach $word2(keys %$hrefTargethash)
            {
                
		if(exists $targetHash{$word2})
                {
                    if(!(exists $temphash{$word2}))
		    {
			$refTargetwords->{$word2}++;
		    }
                    $temphash{$word2}++;	
                    $pair = $word1." ".$word2;
                    $refPairs->{$pair}++;
                    
                 }
	     }
	  }

     }  

}


# This functions takes hash as input which contains all the pairs. A hash with
# source words above cutoff frequency as key and value as the number of pieces
# in which that word has occurred. Similar hash for target words. Then it 
# prints all the word pairs, number of pieces in which they have occurred 
# together, number of pieces in which first word of the pair has occurred and
# number of pieces in which the second word has occurred.
sub printTofile{

    my $refpairs = $_[0];
    my $refSource = $_[1];
    my $refTarget = $_[2];
    my $pairs; 

%$refpairs = &deleteHash(%$refpairs);
    $pieces =~ s/\s+//g;
    $pieces = $pieces;
    print "$pieces";
  foreach $pairs(keys %$refpairs)
  {
      ($word1, $word2) = split(/ /,$pairs);
print  "\n$word1<>$word2<>$refpairs->{$pairs} $refSource->{$word1} $refTarget->{$word2}";
   }

    print "\n";
}	 














